// Implementation of the Lookup Service interface used to send call information
// to the client.
// Authors:
// Cameron Hayes
// Mac ...
// Elias Sarraf
unit Lookup.ServiceImpl;

interface

uses
  XData.Server.Module,
  XData.Service.Common,
  Database, Data.DB,
  Lookup.Service, System.Hash, System.Classes;


type
  [ServiceImplementation]
  TLookupService = class(TInterfacedObject, ILookupService)
  strict private
  callsDB: TFDatabaseModule;
  private
    function GetItems(searchOptions: string): TItemList;
    function GetUsers(searchOptions: string): TUserList;
    function GetCalls(searchOptions: string): TCallList;
    function EditUser(const editOptions: string): string;
    function Search(phoneNum: string): TCallList;
    function AddUser(userInfo: string): string;
    function AddItem(itemInfo: string): string;
    function DelUser(username: string): string;
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
  end;

implementation

uses
  System.SysUtils,
  System.Generics.Collections,
  XData.Sys.Exceptions, uLibrary;

procedure TLookupService.AfterConstruction;
begin
  inherited;
  callsDB := TFDatabaseModule.Create(nil);

end;

procedure TLookupService.BeforeDestruction;
begin
  callsDB.Free;
  inherited;
end;

function TLookupService.Search(phoneNum: string): TCallList;
// Searchs the database for a specific from phone number then returns a list of
// calls from said number
// SQL: String of the sql query used to retrieve info from the database
// call: object representing the call information we want to send to the client
// Return: A list of calls
var
SQL: string;
call: TCallItem;
begin
  SQL := 'select * ' +
          'from calls inner join recordings on calls.sid = recordings.call_sid ' +
         'where from_formatted = ' + QuotedStr(PhoneNum)  + ' order by calls.date_created desc';

  doQuery(callsDB.UniQuery1, SQL);

  Result:= TCallList.Create;
  Result.data := TList<TCallItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );

  while not callsDB.UniQuery1.Eof do
    begin
      call := TCallItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( call );
      Result.data.Add( call );
      call.callSid := callsDB.UniQuery1.FieldByName('sid').AsString;
      call.fromNumber := callsDB.UniQuery1.FieldByName('from_formatted').AsString;
      call.ToNumber := callsDB.UniQuery1.FieldByName('to_formatted').AsString;
      call.dateCreated := callsDB.UniQuery1.FieldByName('date_created').AsString;
      call.mediaUrl := callsDB.UniQuery1.FieldByName('media_url').AsString;
      call.duration := callsDB.UniQuery1.FieldByName('duration').AsString;
      call.transcription := callsDB.UniQuery1.FieldByName('transcription').AsString;
      callsDB.UniQuery1.Next;
    end;
    callsDB.UniQuery1.Close;
    SQL:= 'select count(*) as total_count from calls inner join ' +
    'recordings on calls.sid = recordings.call_sid where ' +
    'from_formatted = ' + QuotedStr(PhoneNum);

    doQuery(callsDB.UniQuery1, SQL);

    Result.count := callsDB.UniQuery1.FieldByName('total_count').AsInteger;

end;

function TLookupService.GetCalls(searchOptions: string): TCallList;
// Retrieves a list of calls from the database then sends it to the client.
// searchOptions: csv string representing all the search options the client sent.
// Format: Phone Number, Page Number, Page Size, Start Date, End Date, SortBy.
// Returns a TCallList containing PageSize amount of entries.
var
params: TStringList;
SQL: string;
DBString: string;
call: TCallItem;
offset: string;
limit: string;
PhoneNum: string;
PageNum: integer;
PageSize: integer;
StartDate: string;
EndDate: string;
OrderBy: string;
whereSQL: string;
orderBySQL: string;
Caller: string;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := searchOptions;
  PhoneNum := params.Values['phonenumber'];
  PageNum := StrToInt(params.Values['pagenumber']);
  PageSize := StrToInt(params.Values['pagesize']);
  StartDate := params.Values['startdate'];
  EndDate := params.Values['enddate'];
  OrderBy := params.Values['orderby'];
  Caller := params.Values['caller'];

  // We want entries for the "page" the client is on.
  offset := IntToStr((PageNum - 1) * PageSize);
  limit := IntToStr(PageSize);

  whereSQL := 'where ';

  if PhoneNum <> '' then
    whereSQL := whereSQL + 'to_formatted = ' + QuotedStr(PhoneNum);

  if StartDate <> '' then
    if whereSQL = 'where ' then
      whereSQL := whereSQL + 'calls.date_created > ' + QuotedStr(StartDate)
    else
      whereSQL := whereSQL + 'AND calls.date_created > ' + QuotedStr(StartDate);

  if EndDate <> '' then
    if whereSQL = 'where ' then
      whereSQL := whereSQL + 'calls.date_created < ' + QuotedStr(EndDate)
    else
      whereSQL := whereSQL + 'AND calls.date_created < ' + QuotedStr(EndDate);

  if Caller <> '' then
    if whereSQL = 'where ' then
      whereSQL := whereSQL + 'from_formatted = ' + QuotedStr(Caller)
    else
      whereSQL := whereSQL + 'AND from_formatted = ' + QuotedStr(Caller);

  if whereSQL = 'where ' then
    whereSQL := '';

  if (OrderBy = '') or (OrderBy = 'Date') then
    orderBySQL :=  'order by calls.date_created desc'
  else
    orderBySQL := 'order by calls.from_formatted desc';

  //this SQL will enventually be modifed for more generic work
  SQL := 'select * ' +
          'from calls inner join recordings on calls.sid = recordings.call_sid ' +
          whereSQL + ' ' + orderBySQL + ' ' + 'limit ' + limit + ' offset ' + offset;

  doQuery(callsDB.UniQuery1, SQL);

  Result:= TCallList.Create;
  Result.data := TList<TCallItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );

  while not callsDB.UniQuery1.Eof do
    begin
      call := TCallItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( call );
      Result.data.Add( call );
      call.callSid := callsDB.UniQuery1.FieldByName('sid').AsString;
      call.fromNumber := callsDB.UniQuery1.FieldByName('from_formatted').AsString;
      call.ToNumber := callsDB.UniQuery1.FieldByName('to_formatted').AsString;
      call.dateCreated := callsDB.UniQuery1.FieldByName('date_created').AsString;
      call.mediaUrl := callsDB.UniQuery1.FieldByName('media_url').AsString;
      call.duration := callsDB.UniQuery1.FieldByName('duration').AsString;
      call.transcription := callsDB.UniQuery1.FieldByName('transcription').AsString;
      callsDB.UniQuery1.Next;
    end;
    callsDB.UniQuery1.Close;
    SQL:= 'select count(*) as total_count from calls inner join recordings on calls.sid = recordings.call_sid ' +
          whereSQL;
    doQuery(callsDB.UniQuery1, SQL);
    Result.count := callsDB.UniQuery1.FieldByName('total_count').AsInteger;
    callsDB.UniQuery1.Close;
end;

function TLookupService.GetItems(searchOptions: string): TItemList;
var
  params: TStringList;
  PageNum: integer;
  PageSize: integer;
  OrderBy: string;
  offset: string;
  limit: string;
  SQL: string;
  item: TItemItem;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := searchOptions;

  PageNum := StrToInt(params.Values['pagenumber']);
  PageSize := StrToInt(params.Values['pagesize']);
  OrderBy := params.Values['orderby'];

  offset := IntToStr((PageNum - 1) * PageSize);
  limit := IntToStr(PageSize);

  SQL := 'select * from qb_items ' + 'limit ' + limit + ' offset ' + offset;

  doQuery(callsDB.UniQuery1, SQL);

  Result:= TItemList.Create;
  Result.data := TList<TItemItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );

  while not callsDB.UniQuery1.Eof do
    begin
      item := TItemItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( item );
      Result.data.Add( item );
      item.ID := callsDB.UniQuery1.FieldByName('qb_items_id').AsString;
      item.name := callsDB.UniQuery1.FieldByName('qb_item_name').AsString;
      item.description := callsDB.UniQuery1.FieldByName('item_desc').AsString;
      item.status := callsDB.UniQuery1.FieldByName('status').AsString;

      callsDB.UniQuery1.Next;
    end;
    callsDB.UniQuery1.Close;
    SQL:= 'select count(*) as total_count from qb_items';
    doQuery(callsDB.UniQuery1, SQL);
    Result.count := callsDB.UniQuery1.FieldByName('total_count').AsInteger;
    callsDB.UniQuery1.Close;
end;

function TLookupService.GetUsers(searchOptions: string): TUserList;
var
SQL: string;
user: TUserItem;
begin
  if searchOptions = '' then
    SQL := 'select * from users order by NAME ASC'
  else
    SQL := 'select * from users where username=' + quotedStr(searchOptions);
  doQuery(callsDB.UniQuery1, SQL);

  Result:= TUserList.Create;
  Result.data := TList<TUserItem>.Create;
  TXDataOperationContext.Current.Handler.ManagedObjects.Add( Result.data );

  while not callsDB.UniQuery1.Eof do
    begin
      user := TUserItem.Create;
      TXDataOperationContext.Current.Handler.ManagedObjects.Add( user );
      Result.data.Add( user );
      user.userID := callsDB.UniQuery1.FieldByName('USER_ID').AsString;
      user.username := callsDB.UniQuery1.FieldByName('USER_NAME').AsString;
      user.password := callsDB.UniQuery1.FieldByName('PASSWORD').AsString;
      user.full_name := callsDB.UniQuery1.FieldByName('NAME').AsString;
      user.status := callsDB.UniQuery1.FieldByName('STATUS').AsString;
      user.email_address := callsDB.UniQuery1.FieldByName('EMAIL').AsString;
      user.AType := callsDB.UniQuery1.FieldByName('ACCESS_TYPE').AsString;
      user.rights := callsDB.UniQuery1.FieldByName('SYSTEM_RIGHTS').AsInteger;
      user.perspectiveID := callsDB.UniQuery1.FieldByName('PERSPECTIVE_ID').AsString;
      user.QBID := callsDB.UniQuery1.FieldByName('QB_ID').AsString;
      callsDB.UniQuery1.Next;
    end;
    callsDB.UniQuery1.Close;
    SQL:= 'select count(*) as total_count from users';
    doQuery(callsDB.UniQuery1, SQL);
    Result.count := callsDB.UniQuery1.FieldByName('total_count').AsInteger;
    callsDB.UniQuery1.Close;
end;

function TLookupService.EditUser(const editOptions: string): string;
var
params: TStringList;
user: string;
password: string;
full_name: string;
status: string;
email: string;
access: string;
rights: string;
perspective: string;
QB: string;
SQL: string;
newUser: string;
hashString: string;
hashPW: string;


begin
  params := TStringList.Create;
  // parse the searchOptions
  params.Delimiter := '&';
  params.StrictDelimiter := true;
  params.DelimitedText := editOptions;
  user := params.Values['username'];
  password := params.Values['password'];
  full_name := params.Values['fullname'];
  status := params.Values['status'];
  email := params.Values['email'];
  access := params.Values['access'];
  rights := params.Values['rights'];
  perspective := params.Values['perspective'];
  QB := params.Values['QB'];
  newUser := params.Values['newuser'];

  SQL := 'select * from users where USER_NAME = ' + QuotedStr(user);
  doQuery(callsDB.UniQuery1, SQL);

  if callsDB.UniQuery1.IsEmpty then
    Result := 'Failure:No such user found'
  else
  begin
    callsDB.UniQuery1.Edit;

    //user.password := callsDB.UniQuery1.FieldByName('PASSWORD').AsString;

    if (not (newUser.IsEmpty)) then
      callsDB.UniQuery1.FieldByName('USER_NAME').AsString := newUser;

    if (not (full_name.IsEmpty)) then
      callsDB.UniQuery1.FieldByName('NAME').AsString := full_name;

    if (not (status.IsEmpty)) then
    begin
      if( StrToBool(status) ) then
        callsDB.UniQuery1.FieldByName('STATUS').AsString := 'ACTIVE'
      else
        callsDB.UniQuery1.FieldByName('STATUS').AsString := 'INACTIVE'
    end;

    if (not (email.IsEmpty)) then
      callsDB.UniQuery1.FieldByName('EMAIL').AsString := email;

    if (not (access.IsEmpty)) then
      callsDB.UniQuery1.FieldByName('ACCESS_TYPE').AsString := Access;

    if (not (rights.IsEmpty)) then
      callsDB.UniQuery1.FieldByName('SYSTEM_RIGHTS').AsInteger := StrToInt(rights);

    if (not (perspective.IsEmpty)) then
      callsDB.UniQuery1.FieldByName('PERSPECTIVE_ID').AsString := perspective;

    if (not (QB.IsEmpty)) then
      callsDB.UniQuery1.FieldByName('QB_ID').AsString := QB;


    {if((not (Password = 'hidden')) and (not (Password.IsEmpty))) then
    begin
      hashString :=  callsDB.UniQuery1.FieldByName('date_created').AsString + password;
      hashPW := THashSHA2.GetHashString(hashString, THashSHA2.TSHA2Version.SHA512).ToUpper;
      callsDB.UniQuery1.FieldByName('password').AsString := hashPW;
    end;}

    callsDB.UniQuery1.Post;
    Result := 'Success:Edit Successful';
  end;
  callsDB.UniQuery1.Close;
end;


function TLookupService.AddUser(userInfo:string): string;
var
user: string;
password: string;
full_name: string;
status: string;
email: string;
access: string;
rights: string;
perspective: string;
QB: string;
SQL: string;
dateCreated: TDateTime;
hashString: string;
hashPW: string;
params: TStringList;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := userInfo;
  dateCreated := now;
  user := params.Values['username'];
  password := params.Values['password'];
  full_name := params.Values['fullname'];
  status := params.Values['status'];
  email := params.Values['email'];
  access := params.Values['access'];
  rights := params.Values['rights'];
  perspective := params.Values['perspective'];
  QB := params.Values['QB'];
  //newUser := params.Values['newuser'];
  //hashString :=  DateTimeToStr(dateCreated) + params.Values['password'];
  //hashPW := THashSHA2.GetHashString(hashString, THashSHA2.TSHA2Version.SHA512).ToUpper;
  SQL := 'select * from users where USER_NAME = ' + QuotedStr(params.Values['username'].toLower);
  callsDB.UniQuery1.Close;
  callsDB.UniQuery1.SQL.Text := SQL;
  callsDB.UniQuery1.Open;
  if callsDB.UniQuery1.IsEmpty then
  begin
    callsDB.UniQuery1.Insert;

    callsDB.UniQuery1.FieldByName('USER_NAME').AsString := user;
    callsDB.UniQuery1.FieldByName('PASSWORD').AsString := password;
    //THashSHA2.GetHashString(hashString, THashSHA2.TSHA2Version.SHA512).ToUpper;
    //callsDB.UniQuery1.FieldByName('date_created').AsString := DateTimeToStr(dateCreated);
    callsDB.UniQuery1.FieldByName('NAME').AsString := full_name;

    if( StrToBool(status) ) then
      callsDB.UniQuery1.FieldByName('STATUS').AsString := 'ACTIVE'
    else
      callsDB.UniQuery1.FieldByName('STATUS').AsString := 'INACTIVE';

    callsDB.UniQuery1.FieldByName('EMAIL').AsString := email;
    callsDB.UniQuery1.FieldByName('ACCESS_TYPE').AsString := Access;
    callsDB.UniQuery1.FieldByName('SYSTEM_RIGHTS').AsInteger := StrToInt(rights);
    callsDB.UniQuery1.FieldByName('PERSPECTIVE_ID').AsString := perspective;
    callsDB.UniQuery1.FieldByName('QB_ID').AsString := QB;

    callsDB.UniQuery1.Post;
    Result := 'Success:User successfully added';
  end
  else
    Result := 'Failure:Username already taken';
end;

function TLookupService.AddItem(itemInfo: string): string;
var
params: TStringList;
Name: string;
Description: string;
Status: boolean;
SQL: string;
begin
  params := TStringList.Create;
  params.StrictDelimiter := true;
  // parse the searchOptions
  params.Delimiter := '&';
  params.DelimitedText := itemInfo;

  Name := params.Values['name'];
  Description := params.Values['description'];
  Status := StrToBool(params.Values['status']);

  SQL := 'select * from qb_items where qb_item_name = ' + QuotedStr(Name);

  doQuery(callsDB.UniQuery1, SQL);
  if callsDB.UniQuery1.IsEmpty then
  begin
    callsDB.UniQuery1.FieldByName('qb_item_name').AsString := Name;
    callsDB.UniQuery1.FieldByName('item_desc').AsString := Description;
    if Status then
      callsDB.UniQuery1.FieldByName('status').AsString := 'ACTIVE'
    else
      callsDB.UniQuery1.FieldByName('status').AsString := 'INACTIVE';

    callsDB.UniQuery1.Post;
    Result := 'Success:Item successfully added';
  end
  else
    Result := 'Failure: Item already exists';
end;

function TLookupService.DelUser(username: string): string;
var

SQL: string;
params: TStringList;
begin
  SQL := 'select * from users where username = ' + QuotedStr(username.toLower);
  callsDB.UniQuery1.Close;
  callsDB.UniQuery1.SQL.Text := SQL;
  callsDB.UniQuery1.Open;
  if callsDB.UniQuery1.IsEmpty then
  begin
    Result := 'Failure:User does not exist';
  end
  else
  begin
    SQL:= 'DELETE FROM users where username = ' +  QuotedStr(username.toLower);
    callsDB.UniQuery1.SQL.Text := SQL;
    callsDB.UniQuery1.ExecSQL;
    Result := 'Success:User deleted';
  end;
end;

initialization
  RegisterServiceType(TLookupService);

end.

